import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ExtensibleURIConverterImpl;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

/**
 * Creating all resources with relative paths. These relative URIs will be normalized by the
 * provided custom URIConverter to absolute ones during serialization/deserialization.
 * However, this normalization does not take place when the schemaLocations are generated.
 * 
 * Resulting problem:
 * The optmisedModel.xmi will use the relative URI of the meta model as the reference used in
 * it schemaLocation (xsi:schemaLocation="http://www.minimalExample.com metaModel.ecore").
 * Therefore, it can not be opened directly in the Sample Reflective Ecore Model Editor.
 */
public class Problem {
	
	public static void main(String[] args) {
		// Create ResourceSet and register factories
		ResourceSet rs = new ResourceSetImpl();
		Map<String, Object> map = rs.getResourceFactoryRegistry().getExtensionToFactoryMap();
		map.put("ecore", new EcoreResourceFactoryImpl());
		map.put("xmi", new XMIResourceFactoryImpl());
		
		// Define a base path and allow the ResourceSet to resolve relative URIs against this base path 
		String basePath = new File("input").getAbsolutePath() + File.separatorChar;
		URI baseUri = URI.createFileURI(basePath);	
		rs.setURIConverter(new ExtensibleURIConverterImpl() {
			@Override
			public URI normalize(URI uri) {
				if (uri.isFile() && uri.isRelative() && baseUri!=null) {
					return uri.resolve(baseUri);
				} else {
					return super.normalize(uri);
				}
			}	
		});
		
		// Load the meta model dynamically and register it in the package registry of the ResourceSet 
		URI ecoreUri = URI.createFileURI("metaModel.ecore");
		Resource rMeta = rs.getResource(ecoreUri, true);
		EPackage metamodel = (EPackage)rMeta.getContents().get(0);
		rs.getPackageRegistry().put(metamodel.getNsURI(), metamodel);
		
		// Load the input instance model dynamically
		URI inputUri = URI.createFileURI("instanceModel.xmi");
		Resource rInput = rs.getResource(inputUri, true);
		EObject inputRoot = rInput.getContents().get(0);
		
		// ... Optimisation would take place here ...
		
		// Create an output resource for the optimised model instance and serialize it
		URI outputUri = URI.createFileURI("../results/run1/optimisedModel.xmi");
		Resource rOut = rs.createResource(outputUri);
		rOut.getContents().clear();
		rOut.getContents().add(inputRoot);

		Map<Object,Object> options = new HashMap<Object,Object>();		
		options.put(XMIResource.OPTION_SCHEMA_LOCATION, Boolean.TRUE);
		
		try {
			rOut.save(options);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
}
